/*
   Denial-Of-Service firewall
   Copyright (c) 1999  Martin Hinner <mhi@penguin.cz>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <linux/kernel.h>

#include <linux/ip.h>
#include <linux/udp.h>

#include "dosfw.h"

int echoloop_attack(struct sk_buff *skb);

struct dosfw_hook echoloop_hook = {
	NULL,
	echoloop_attack,
	"echoloop"
};

void register_echoloop()
{
	dosfw_register(&echoloop_hook);
}

int echoloop_attack(struct sk_buff *skb)
{
	const struct iphdr *ip = skb->nh.iph;
	struct udphdr *udp;

	if (ip->protocol != IPPROTO_UDP)
		return DOSFW_ACCEPT;

 
	udp = (struct udphdr *)((__u32 *)ip+ip->ihl);

 	if ( ! ( ( ntohs(udp->source)==19 || ntohs(udp->source)==7) && 
               ( ntohs(udp->dest)==19 || ntohs(udp->dest)==7) ) ) 
		return DOSFW_ACCEPT;

	printk("[dosfw] Echo loop attack detected!"
               " [%ld.%ld.%ld.%ld:%hu -> %ld.%ld.%ld.%ld:%hu]\n",
		(ntohl(ip->saddr)>>24)&0xFF,
		(ntohl(ip->saddr)>>16)&0xFF,
		(ntohl(ip->saddr)>>8)&0xFF,
		(ntohl(ip->saddr))&0xFF,
		ntohs(udp->source),
		(ntohl(ip->daddr)>>24)&0xFF,
		(ntohl(ip->daddr)>>16)&0xFF,
		(ntohl(ip->daddr)>>8)&0xFF,
		(ntohl(ip->daddr))&0xFF,
		ntohs(udp->dest));
		
	return DOSFW_REJECT;
}
