/*
   Denial-Of-Service firewall
   Copyright (c) 1999  Martin Hinner <mhi@penguin.cz>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <linux/kernel.h>

#include <linux/ip.h>
#include <linux/tcp.h>

#include "dosfw.h"

int fingerprint_scan(struct sk_buff *skb);

struct dosfw_hook fingerprint_hook = {
	NULL,
	fingerprint_scan,
	"fingerprint"
};

void register_fingerprint()
{
	dosfw_register(&fingerprint_hook);
}

int fingerprint_scan(struct sk_buff *skb)
{
	const struct iphdr *ip = skb->nh.iph;
	struct tcphdr *tcp;
	char *reason;

	reason = 0;

	if (ip->protocol != IPPROTO_TCP)
		return DOSFW_ACCEPT;
 
	tcp = (struct tcphdr *)((__u32 *)ip+ip->ihl);

	if (tcp->res2)
		reason = "reserved TCP flags are set";

	if (!reason)
		return DOSFW_ACCEPT;

	printk("[dosfw] Fingerprint scan detected, %s [scan of %d.%d.%d.%d"
               " from %ld.%ld.%ld.%ld].\n",
		reason,
		(ntohl(ip->daddr)>>24)&0xFF,
		(ntohl(ip->daddr)>>16)&0xFF,
		(ntohl(ip->daddr)>>8)&0xFF,
		(ntohl(ip->daddr))&0xFF,

		(ntohl(ip->saddr)>>24)&0xFF,
		(ntohl(ip->saddr)>>16)&0xFF,
		(ntohl(ip->saddr)>>8)&0xFF,
		(ntohl(ip->saddr))&0xFF);
		
	return DOSFW_REJECT;
}
