/*
   Denial-Of-Service firewall
   Copyright (c) 1999  Martin Hinner <mhi@penguin.cz>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <linux/module.h>
#include <linux/kernel.h>

#include <linux/socket.h>
#include <linux/in.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/string.h>
#include <linux/sockios.h>
#include <linux/net.h>
#include <linux/fcntl.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/list.h>

#include <linux/ip.h>

#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>

#include "dosfw.h"
#include "config.h"

struct dosfw_hook *dosfw_hooks;
static rwlock_t dosfw_lock = RW_LOCK_UNLOCKED;

int dosfw_register(struct dosfw_hook *hook)
{
        write_lock_bh(&dosfw_lock);
	hook->next = dosfw_hooks;
	dosfw_hooks = hook;
        write_unlock_bh(&dosfw_lock);
	return 0;
}

int process_skb(struct sk_buff *skb)
{
	struct dosfw_hook *h;

        read_lock_bh(&dosfw_lock);
	h = dosfw_hooks;

	while (h) {
		if (h->hook(skb) != DOSFW_ACCEPT) {
        		read_unlock_bh(&dosfw_lock);
			return DOSFW_REJECT;
		}
		h = h->next;
	}

 	read_unlock_bh(&dosfw_lock);
	return DOSFW_ACCEPT;
}

unsigned int nf_doshook(unsigned int hooknum,
                        struct sk_buff **skb,
                        const struct net_device *in,
                        const struct net_device *out)
{
	int rc;

	rc = process_skb(*skb);
	if (rc == DOSFW_ACCEPT)
		return NF_ACCEPT;
	return NF_DROP;
}

struct nf_hook_in dosfw_hook_ops = 
{
	{ NULL, NULL },
	nf_doshook,
	NULL,
	PF_INET,
	NF_IP_LOCAL_IN,
};

struct nf_hook_forward dosfw_hook_ops = 
{
	{ NULL, NULL },
	nf_doshook,
	NULL,
	PF_INET,
	NF_IP_FORWARD
};

int init_module(void)
{
	int rc;

	printk("Denial-Of-Service firewall, version -1L\n");
	dosfw_hooks = 0;

	register_land();
	register_echoloop();
	register_fingerprint();

	rc = nf_register_hook(&dosfw_hook_in);
	if (rc)
		return rc;
	return nf_register_hook(&dosfw_hook_forward);
}

void cleanup_module(void)
{
	nf_unregister_hook(&dosfw_hook_in);
	nf_unregister_hook(&dosfw_hook_forward);
}
